#include "io_lib.h"
#include <avr/io.h>

void IO_init()
{
	// Set input and output directions
	PORTB = 0;
	DDRB = (1<<PB3)|(1<<PB4)|(1<<PB5)|(1<<PB6)|(1<<PB7);
	
	// Setup the PWM module for the servo, based on 4.096MHz clock source
	TCCR1A |= (1<<COM1A1)|(1<<COM1B1)|(1<<WGM11);		 			// NON Inverted PWM
	TCCR1B |= (1<<WGM13)|(1<<WGM12)|(0<<CS12)|(1<<CS11)|(0<<CS10);	// PRESCALER=8 MODE 14(FAST PWM)
	ICR1 = 10240;													// fPWM=50Hz (Period = 20ms Standard).
	
	OCR1A = 640;	// 90 degrees, should be center
	
	// Allow Timer 1 overflow interrupts
	TIMSK = (1 << TOIE1);
}

// Turn the electromagnet on or off
void setSteeringAngle(uint8_t angle)
{
	// Scale from 8 bytes to 9 bytes (256 to 512 capacity)
	uint16_t scaledAngle = (angle << 1);
	
	// Pulse lengths from 1000s to 2000s defining -90 to +90 degrees
	OCR1A = 512 + scaledAngle;
}

void setMotorPowerAndDirection(uint8_t duty, uint8_t direction)
{
	if (duty > 0)
	{
		if (direction > 0)
		{
			PORTD &=~(1<<PD4);
			PORTD |= (1<<PD5);
		}
		else
		{
			PORTD |= (1<<PD4);
			PORTD &=~(1<<PD5);
		}
	}
	else
	{
		PORTD &=~((1<<PD4)|(1<<PD5));
	}
}

void setHeadLights(uint8_t intensity)
{
	if (intensity)
	{
		PORTD |= (1<<PD2);
	}
	else
	{
		PORTD &=~(1<<PB2);
	}
}

void setBrakeLights(uint8_t intensity)
{
	if (intensity)
	{
		PORTD |= (1<<PB3);
	}
	else
	{
		PORTD &=~(1<<PB3);
	}
}
