#include "usart_lib.h"
#include <avr/io.h>
#include <avr/interrupt.h>

// Initialize USART Module
void USART0_init(void)
{
	// Load upper 8-bits of the baud rate value into the high byte of the UBRR register
	UBRRH = (BAUD_PRESCALE >> 8);
	// Load lower 8-bits of the baud rate value into the low byte of the UBRR register
	UBRRL = BAUD_PRESCALE;

	//Enable USART transceiver and interrupt on reception
	UCSRB = (1<<RXCIE)|(1<<RXEN)|(1<<TXEN);
	//Set data frame format: asynchronous mode, no parity, 1 stop bit, 8 bit size
	UCSRC = (1<<UCSZ1)|(1<<UCSZ0);
	
	// Initialize the buffer
	writeIndex = 0;
	currentIndex = 0;
}

// Send a single byte over USART
void USART0_send_byte(uint8_t data)
{
	// Wait until buffer is empty (UDRE0 set)
	while (!(UCSRA & (1<<UDRE)));
	
	// Place data in the out register
	UDR = data;
}

uint8_t popFromBuffer(void)
{
	uint8_t temp;
	
	// Pop a byte from the stack, in an atomic action
	cli();
	
	temp = usartBuffer[currentIndex];
	
	currentIndex++;
	if (currentIndex >= BUFFERSIZE)
	{
		currentIndex = 0;
	}
	
	sei();
	
	return temp;
}

void bufferInsert(uint8_t data)
{
	// This must be done atomically
	cli();
	
	usartBuffer[writeIndex] = data;
	
	writeIndex++;
	if (writeIndex >= BUFFERSIZE)
	{
		writeIndex = 0;
	}
	
	sei();
}

// Send a 16-bit word over UART, with a 8-bit code first
void USART0_send_word(uint8_t code, int16_t data)
{
	union SerialSignedWord temp;
	temp.word = data;
	
	// First the type of data packet
	USART0_send_byte(code);
	// Then data
	USART0_send_byte(temp.bytes[0]);
	USART0_send_byte(temp.bytes[1]);
}

void USART0_send_float(uint8_t code, float data)
{
	union SerialFloat temp;
	temp.f = data;
	
	// First the type of data packet
	USART0_send_byte(code);
	// Then data
	USART0_send_byte(temp.bytes[0]);
	USART0_send_byte(temp.bytes[1]);
	USART0_send_byte(temp.bytes[2]);
	USART0_send_byte(temp.bytes[3]);
}
